/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language2.*;

/**
   This program parses and then evaluates strings from Language_2.
*/
public class Language_2_Examples
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      EvaluateAST.DEBUG = 1;

      String[] programs =
      {
         "(print x)",                    // invalid expression
         "(+ x 2)",                      // invalid expression
         "(prog (print x) (var x 0))",   // invalid expression
         "(prog (var x 0) (&& true x))", // invalid expression
         "(prog (var x true) (* 5 x))",  // invalid expression
         "(+ (== (+ 2 2) 4) 0)",         // invalid expression

         "(var x (var x 2))",

         String.join("\n",
         "(prog ",
         "   (var x 2) ",
         "   (var y x))"),

         String.join("\n",
         "(prog ",
         "   (var x 2) ",
         "   (var y 2) ",
         "   (var z (+ x y)))"),

         String.join("\n",
         "(prog ",
         "   (var x 0) ",
         "   (var y 0) ",
         "   (var x (+ x 1)) ",
         "   (var y (+ x y)) ",
         "   (var x (+ x y 3)))"),

         String.join("\n",
         "(prog ",
         "   (var x 4) ",
         "   (var y false) ",
         "   (|| y (== (+ 2 x) (* 2 3))))"),

         "(var z (+ (var x (var y 2)) (+ y (* 3 x))))",

         String.join("\n",
         "(prog ",
         "   (var x 1) ",
         "   (+ (* 3 (var x (+ x 1))) (* 2 (var x (+ x 1)))))"),

         String.join("\n",
         "(prog ",
         "   (var n 1) ",
         "   (var x1 (* n n)) ",
         "   (var n 2) ",
         "   (var x2 (* n n)) ",
         "   (var n 3) ",
         "   (var x3 (* n n)) ",
         "   (var n 4) ",
         "   (var x4 (* n n)) ",
         "   (+ (+ (+ x1 x2) x3) x4) ",
         "   (+ x1 x2 x3 x4))"),

         String.join("\n",
         "(prog ",
         "   (var n 1) ",
         "   (var x (* n n)) ",
         "   (print x) ",
         "   (var n 2) ",
         "   (var x (+ x (* n n))) ",
         "   (print x) ",
         "   (var n 3) ",
         "   (var x (+ x (* n n))) ",
         "   (print x) ",
         "   (var n 4) ",
         "   (+ x (* n n)))")
      };

      int i = 0;
      for (final String str : programs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the program.
         try
         {
            final Tree<Token> ast = Parse2AST.parse( str );

            // Print the AST as an S-expression
            System.out.println( EvaluateAST.toString(ast) + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.println("// Unparsed program: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_2.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            // Run the program (interpret the AST).
            final Value value = EvaluateAST.eval( ast );
            if (EvaluateAST.DEBUG > 0)
            {
               System.out.println("result = " + value + "\n" );
            }
            else
            {
               System.out.println("result = " + value.toSimpleString() + "\n" );
            }
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
