/*

*/

package language2;

import java.util.ArrayList;

/**
   An {@code Environment} object holds (variable, value) pairs.
   <p>
   The {@code Environment} interface has methods for
   <ol>
   <li> Adding a new (variable, value) pair to this {@code Environment} object.</li>
   <li> Looking up a variable to see if it is part of a (variable, value) pair.</li>
   <li> Looking up a variable's value from the variable's (variable, value) pair.</li>
   <li> Mutating the value part of a variable's (variable, value) pair.</li>
   </ol>
*/
public class Environment
{
   private final ArrayList<String> variables;
   private final ArrayList<Value>  values;

   /**
      Create an empty {@code Environment} object.
   */
   public Environment()
   {
      variables = new ArrayList<String>();
      values    = new ArrayList<Value>();
   }


   /**
      Look up {@code variable} in this {@code Environment} object.
      Return true if {@code variable} is in this {@code Environment}
      object, otherwise return false.

      @param variable  to be looked up in this {@code Environment} object
      @return true if the variable is found in this {@code Environment} object
   */
   public boolean defined(String variable)
   {
      return ( null != this.lookUp(variable) );
   }


   /**
      Look up {@code variable} in this {@code Environment}
      object and return its associated {@link Value}.
      <p>
      Return null if {@code variable} is not found in this
      {@code Environment} object.

      @param variable  to be looked up in this {@code Environment} object
      @return the {@link Value} of the variable, if it is found in this {@code Environment} object
   */
   public Value lookUp(String variable)
   {
      int i;
      for (i = 0; i < variables.size(); ++i)
         if ( variable.equals(variables.get(i)) )
            break;

      if ( i < variables.size() )
      {
         return values.get(i);
      }
      else
      {
         return null;  // variable cannot be found
      }
   }


   /**
      Add a (variable, value) pair to this {@code Environment} object.

      @param variable to be added to this {@code Environment}
      @param value {@link Value} object to associate with the given variable
   */
   public void add(String variable, Value value)
   {
      variables.add(variable);
         values.add(value);
   }


   /**
      Update the {@link Value} associated with {@code variable}
      in this {@code Environment} object. Return true if the update
      is successful, return false if {@code variable} is not found
      in this {@code Environment} object.

      @param variable  to be updated in this {@code Environment} object
      @param value  {@link Value} to put in this {@code Environment} for the given variable
      @return true if the variable was found and updated
   */
   public boolean update(String variable, Value value)
   {
      int i;
      for (i = 0; i < variables.size(); ++i)
         if ( variable.equals(variables.get(i)) )
            break;

      if ( i < variables.size() )
      {
         values.set(i, value);
         return true;
      }
      else
      {
         return false;
      }
   }


   @Override
   public String toString()
   {
      String result = "";

      result += "[Global Environment";

      for (int i = 0; i < variables.size(); ++i)
      {
         result += "\n " + variables.get(i) + " = " + values.get(i);
      }
      result += "]";

      return result;
   }
}
